@@ -27,12 +27,6 @@ void kissat_decrease_phases (struct kissat *, unsigned);
 void kissat_release_phases (struct kissat *);
 
 void kissat_save_best_phases (struct kissat *);
-void kissat_save_saved_phases (struct kissat *);
 void kissat_save_target_phases (struct kissat *);
 
-#define all_phases(NAME, PTR) \
-  value *PTR = solver->phases.NAME, *const end_##PTR = PTR + VARS; \
-  PTR != end_##PTR; \
-  ++PTR
-
 #endif
